@@ -37,38 +37,36 @@ except (ImportError, SyntaxError, ):
 
 
 HEADER = {'User-Agent': 'Mozilla/5.1 (X11; U; Linux i686; en-US; rv:1.8.0.3) Gecko/20060425 SUSE/1.5.0.3-7 Hv3/alpha'}
+BUILTINPAGES = {"about:blank": "<html><body></body></html>",
+                 "about:tkinterweb": "<html><head><title>about:tkinterweb</title></head><body>Maybe someday this page will become something interesting. For now it's totally useless. Good job finding it though.</body></html>"}
 DEFAULTSTYLE = """
 /* Default stylesheet to be loaded whenever HTML is parsed. */
 /* This is a modified version of the stylesheet that comes bundled with Tkhtml. */
-
 /* Display types for non-table items. */
   ADDRESS, BLOCKQUOTE, BODY, DD, DIV, DL, DT, FIELDSET, 
   FRAME, H1, H2, H3, H4, H5, H6, NOFRAMES, 
   OL, P, UL, APPLET, CENTER, DIR, HR, MENU, PRE, FORM
                 { display: block }
-
 HEAD, SCRIPT, TITLE { display: none }
 BODY {
   margin:8px;
 }
-
-/* Rules for unordered-lists */
+/* Rules for lists */
 LI                   { display: list-item }
+OL, UL, DIR, MENU, DD  { padding-left: 40px ; margin-left: 1em }
+OL[type]         { list-style-type : tcl(::tkhtml::ol_liststyletype) }
+UL>LI { list-style-type : disc }
+UL>UL>LI { list-style-type : circle }
+UL>UL UL>LI { list-style-type : square }
 UL[type="square"]>LI { list-style-type : square } 
 UL[type="disc"]>LI   { list-style-type : disc   } 
 UL[type="circle"]>LI { list-style-type : circle } 
 LI[type="circle"]    { list-style-type : circle }
 LI[type="square"]    { list-style-type : square }
 LI[type="disc"]      { list-style-type : disc   }
-
-OL, UL, DIR, MENU, DD  { padding-left: 40px ; margin-left: 1em }
-
-OL[type]         { list-style-type : tcl(::tkhtml::ol_liststyletype) }
-
 NOBR {
   white-space: nowrap;
 }
-
 /* Map the 'align' attribute to the 'float' property. Todo: This should
  * only be done for images, tables etc. "align" can mean different things
  * for different elements.
@@ -85,7 +83,6 @@ TABLE[align="center"]     {
 }
 IMG[align="left"]         { float:left }
 IMG[align="right"]        { float:right }
-
 /* If the 'align' attribute was not mapped to float by the rules above, map
  * it to 'text-align'. The rules above take precedence because of their
  * higher specificity. 
@@ -95,10 +92,8 @@ IMG[align="right"]        { float:right }
 [align="right"]              { text-align: -tkhtml-right }
 [align="left"]               { text-align: -tkhtml-left  }
 CENTER, [align="center"]     { text-align: -tkhtml-center }
-
 /* Rules for unordered-lists */
 /* Todo! */
-
 TD, TH {
   padding: 1px;
   border-bottom-color: grey60;
@@ -106,7 +101,6 @@ TD, TH {
   border-top-color: grey25;
   border-left-color: grey25;
 }
-
 /* For a horizontal line, use a table with no content. We use a table
  * instead of a block because tables are laid out around floating boxes, 
  * whereas regular blocks are not.
@@ -122,27 +116,22 @@ HR {
   margin: 0.5em 0;
 }
 */
-
 HR {
   display: block;
   border-top:    1px solid grey45;
   border-bottom: 1px solid grey80;
   margin: 0.5em auto 0.5em auto;
 }
-
 /* Basic table tag rules. */
 TABLE { 
   display: table;
   border-spacing: 0px;
-
   border-bottom-color: grey25;
   border-right-color: grey25;
   border-top-color: grey60;
   border-left-color: grey60;
-
   text-align: left;
 }
-
 TR              { display: table-row }
 THEAD           { display: table-header-group }
 TBODY           { display: table-row-group }
@@ -153,7 +142,6 @@ TD, TH          { display: table-cell }
 CAPTION         { display: table-caption }
 TH              { font-weight: bolder; text-align: center }
 CAPTION         { text-align: center }
-
 H1              { font-size: 2em; margin: .67em 0 }
 H2              { font-size: 1.5em; margin: .83em 0 }
 H3              { font-size: 1.17em; margin: 1em 0 }
@@ -182,7 +170,6 @@ OL UL, UL OL,
 UL UL, OL OL    { margin-top: 0; margin-bottom: 0 }
 U, INS          { text-decoration: underline }
 ABBR, ACRONYM   { font-variant: small-caps; letter-spacing: 0.1em }
-
 /* Formatting for <pre> etc. */
 PRE, PLAINTEXT, XMP { 
   display: block;
@@ -190,47 +177,38 @@ PRE, PLAINTEXT, XMP {
   margin: 1em 0;
   font-family: courier;
 }
-
 /* Display properties for hyperlinks */
 :link    { color: darkblue; text-decoration: underline ; cursor: pointer }
 :visited { color: purple; text-decoration: underline ; cursor: pointer }
-
 A:active {
     color:red;
     cursor:pointer;
 }
-
 /* Deal with the "nowrap" HTML attribute on table cells. */
 TD[nowrap] ,     TH[nowrap]     { white-space: nowrap; }
 TD[nowrap="0"] , TH[nowrap="0"] { white-space: normal; }
-
 BR { 
     display: block;
 }
 /* BR:before       { content: "\A" } */
-
 /*
  * Default decorations for form items. 
  */
 INPUT[type="hidden"] { display: none }
-
 INPUT, TEXTAREA, SELECT, BUTTON { 
   border: 1px solid black;
   background-color: white;
   line-height: normal;
   vertical-align: middle;
 }
-
 INPUT[type="image"][src] {
   -tkhtml-replacement-image: attr(src);
   cursor: pointer;
 }
-
 INPUT[type="checkbox"], INPUT[type="radio"], input[type="file"], input[type="range"], input[type="color"] {
   background-color: transparent;
   border: none;
 }
-
 INPUT[type="submit"],INPUT[type="button"], INPUT[type="reset"], BUTTON {
   display: -tkhtml-inline-button;
   position: relative;
@@ -249,28 +227,22 @@ INPUT[type="submit"],INPUT[type="button"], INPUT[type="reset"], BUTTON {
   color: #000000;
   color: tcl(::tkhtml::if_disabled #666666 #000000);
 }
-
 INPUT[type="submit"]:after {
   content: "Submit";
 }
-
 INPUT[type="reset"]:after {
   content: "Reset";
 }
-
 INPUT[type="submit"][value]:after,INPUT[type="button"][value]:after, INPUT[type="reset"][value]:after {
   content: attr(value);
 }
-
 INPUT[type="submit"]:hover:active, INPUT[type="reset"]:hover:active,INPUT[type="button"]:hover:active, BUTTON:hover:active {
   border-top-color:    tcl(::tkhtml::if_disabled #e7e9eb #828282);
   border-left-color:   tcl(::tkhtml::if_disabled #e7e9eb #828282);
   border-right-color:  tcl(::tkhtml::if_disabled #828282 #e7e9eb);
   border-bottom-color: tcl(::tkhtml::if_disabled #828282 #e7e9eb);
 }
-
 INPUT[size] { width: tcl(::tkhtml::inputsize_to_css) }
-
 /* Handle "cols" and "rows" on a <textarea> element. By default, use
  * a fixed width font in <textarea> elements.
  */
@@ -279,18 +251,15 @@ TEXTAREA[rows] { height: tcl(::tkhtml::textarea_height) }
 TEXTAREA {
   font-family: fixed;
 }
-
 FRAMESET {
   display: none;
 }
-
 /* Default size for <IFRAME> elements */
 IFRAME {
   width: 300px;
   height: 200px;
   border: 1px solid black;
 }
-
 /*
  *************************************************************************
  * Below this point are stylesheet rules for mapping presentational 
@@ -298,26 +267,21 @@ IFRAME {
  * shouldn't be specified here (in the UA stylesheet), but it doesn't matter
  * in practice. See CSS 2.1 spec for more details.
  */
-
 /* 'color' */
 [color]              { color: attr(color) }
 body a[href]:link    { color: attr(link x body) }
 body a[href]:visited { color: attr(vlink x body) }
-
 /* 'width', 'height', 'background-color' and 'font-size' */
 [width]            { width:            attr(width l) }
 [height]           { height:           attr(height l) }
 basefont[size]     { font-size:        attr(size) }
 font[size]         { font-size:        tcl(::tkhtml::size_to_fontsize) }
 [bgcolor]          { background-color: attr(bgcolor) }
-
 BR[clear]          { clear: attr(clear) }
 BR[clear="all"]    { clear: both; }
-
 /* Standard html <img> tags - replace the node with the image at url $src */
 IMG[src]              { -tkhtml-replacement-image: attr(src) }
 IMG                   { -tkhtml-replacement-image: "" }
-
 /*
  * Properties of table cells (th, td):
  *
@@ -331,7 +295,6 @@ TABLE[border], TABLE[border] TD, TABLE[border] TH {
     border-right-width:   attr(border l table);
     border-bottom-width:  attr(border l table);
     border-left-width:    attr(border l table);
-
     border-top-style:     attr(border x table solid);
     border-right-style:   attr(border x table solid);
     border-bottom-style:  attr(border x table solid);
@@ -352,7 +315,6 @@ TABLE[cellpadding] td, TABLE[cellpadding] th {
 TABLE[cellspacing], table[cellspacing] {
     border-spacing: attr(cellspacing l);
 }
-
 /* Map the valign attribute to the 'vertical-align' property for table 
  * cells. The default value is "middle", or use the actual value of 
  * valign if it is defined.
@@ -360,17 +322,13 @@ TABLE[cellspacing], table[cellspacing] {
 TD,TH                        {vertical-align: middle}
 TR[valign]>TD, TR[valign]>TH {vertical-align: attr(valign x tr)}
 TR>TD[valign], TR>TH[valign] {vertical-align: attr(valign)}
-
-
 /* Support the "text" attribute on the <body> tag */
 body[text]       {color: attr(text)}
-
 /* Allow background images to be specified using the "background" attribute.
  * According to HTML 4.01 this is only allowed for <body> elements, but
  * many websites use it arbitrarily.
  */
 [background] { background-image: attr(background) }
-
 /* The vspace and hspace attributes map to margins for elements of type
  * <IMG>, <OBJECT> and <APPLET> only. Note that this attribute is
  * deprecated in HTML 4.01.
@@ -383,7 +341,6 @@ IMG[hspace], OBJECT[hspace], APPLET[hspace] {
     margin-left: attr(hspace l);
     margin-right: attr(hspace l);
 }
-
 /* marginheight and marginwidth attributes on <BODY> */
 BODY[marginheight] {
   margin-top: attr(marginheight l);
@@ -393,7 +350,6 @@ BODY[marginwidth] {
   margin-left: attr(marginwidth l);
   margin-right: attr(marginwidth l);
 }
-
 SPAN[spancontent]:after {
   content: attr(spancontent);
 }
@@ -623,6 +579,10 @@ def download(url, data=None, method="GET", decode=None):
     "Fetch files."
     "Technically this isn't thread-safe (even though it is being used inside threads by Tkinterweb, "
     "but as long as install_opener() is not called and a string is used as the url parameter we are okay."
+
+    if url in BUILTINPAGES:
+        return BUILTINPAGES[url], url, 'text/html'
+
     thread = threadname()
     url = url.replace(" ", "%20")
     if data and (method == "POST"):
